﻿using Eingabeverwaltung.Metamodell;
using System;
using System.Collections.Generic;

namespace Ueberpruefung.CheckInduction
{
    /// <summary>
    /// abstract class for checker of error 35 and 40 <br></br>
    /// This class is usefull, because both checker need 95% same code and its necessary to have two different checker for
    /// both errors, because error 35 have to influence on the value isProved while error 40 shouldn't.
    /// </summary>
    public abstract class CheckIndPartEquationFixedVars : ElementChecker
    {
        protected readonly Induction parent;
        public CheckIndPartEquationFixedVars(Induction parent) : base()
        {
            this.parent = parent;
        }

        public override bool check(Element e)
        {
            if (parent.indVar == null) return true;
            if (e is InductionCase ic)
            {
                List<Variable> shouldValue = new List<Variable>(parent.fixedVars);
                shouldValue.Remove(parent.indVar); // for the case ind var is fixed
                if (ic.BuildingParameterVariables != null) 
                    foreach(Variable bPV in ic.BuildingParameterVariables)
                        shouldValue.Add(bPV);
                return check(shouldValue, ic.fixedVars, ic.errors);
            }
            else if (e is InductionHypothesis ih)
            {
                List<Variable> shouldValue = new List<Variable>(parent.fixedVars);
                shouldValue.Remove(parent.indVar); // for the case ind var is fixed
                List<Substitution> subs = CheckHelper.findSubstitution(parent.eq, ih.eq, new List<Variable>(), new List<Variable>());
                if (subs != null)
                    foreach (Substitution sub in subs)
                        if (sub.variable == parent.indVar)
                            if (sub.tree is TreeVariable tv) shouldValue.Add(tv.variable); 
                return check(shouldValue, ih.fixedVars, ih.errors);
            }
            else throw new InvalidOperationException();
        }

        protected abstract bool check(List<Variable> shouldValueFixedVars, List<Variable> isValueFixedVars, List<Error> contextErrorList);

        /// <summary>
        /// checks error 35. Removes all should be fixed vars from isValueFixedVars and prints error output in err35Output if result is false.
        /// </summary>
        /// <param name="shouldValueFixedVars"></param>
        /// <param name="isValueFixedVars"></param>
        /// <param name="err35Output"></param>
        /// <returns></returns>
        protected bool check35(List<Variable> shouldValueFixedVars, List<Variable> isValueFixedVars, out string err35Output)
        {
            err35Output = "";
            // 35: each fixed var from ind must be in fixed vars of indCase
            bool result35 = true;
            foreach (Variable indFixedVar in shouldValueFixedVars)
            { // not direct comparison of vars, because can be renamed to different var with same name
                bool found = false;
                foreach (Variable caseFixedVar in isValueFixedVars)
                    if (indFixedVar.name == caseFixedVar.name)
                    { found = true; isValueFixedVars.Remove(caseFixedVar); break; }
                if (!found) err35Output += indFixedVar.name + " ";
                result35 &= found;
            }
            return result35;
        }
    }
}
