﻿using System;
using System.Collections.Generic;
using Eingabeverwaltung;
using Eingabeverwaltung.Metamodell;

namespace Ueberpruefung
{
    /// <summary>
    /// @pre: Needs to be called after Ueberpruefer.analyseUsefulness
    /// </summary>
    class CheckLemmaTaskIsUsed : ElementChecker
    {
        /// <summary>
        /// @pre: Needs to be called after Ueberpruefer.analyseUsefulness
        /// </summary>
        public CheckLemmaTaskIsUsed() { }
        public override bool check(Element e)
        {
            if (e is Container con)
            {
                List<Lemma> mainLemmata = new List<Lemma>();
                // 1. collect all Lemmata, that are main proof for a task
                foreach (Lemma lemma in con.Proof.lemmata)
                    if (lemma.OwnTask != null) mainLemmata.Add(lemma);
                // 2. remove all those Lemmata from collected list that are reachable from main proof
                void RemoveAllUsefulLemmata(List<Lemma> visited, SingleProof proof)
                {
                    void RemoveAllUsefulLemmataTR(List<Lemma> visited, List<Lemma> newLemmata, Transformation tr)
                    {
                        foreach (TransformationStep ts in tr.transformationSteps)
                        {
                            if (ts.rule is Lemma lemma && !visited.Contains(lemma))
                            {
                                visited.Add(lemma);
                                newLemmata.Add(lemma);
                                mainLemmata.Remove(lemma);
                            }
                        }
                    }
                    List<Lemma> newLemmata = new List<Lemma>();
                    if (proof is Transformation tr)
                        RemoveAllUsefulLemmataTR(visited, newLemmata, tr);
                    else if (proof is Induction ind)
                        foreach (InductionCase indC in ind.indCases)
                            RemoveAllUsefulLemmataTR(visited, newLemmata, indC.proof);
                    else throw new KontextException(proof.coords,"Found a proof that is neither Transformation nor Induction!");
                    foreach (Lemma newLemma in newLemmata)
                        RemoveAllUsefulLemmata(visited, newLemma.proof);
                }
                RemoveAllUsefulLemmata(new List<Lemma>(), con.Proof.proof);
                bool result = mainLemmata.Count == 0;
                if (!result)
                    foreach (Lemma lemma in mainLemmata)
                        lemma.errors.Add(new Error(23));
                return result;
            }
            else throw new InvalidOperationException();
        }
    }
}
